/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.jrunit.sample.basic;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;

import junit.framework.TestCase;

/**
 * This is a very simple client for calling upon the server via sockets and just send simple
 * text with the thread id of the client caller.
 *
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class SimpleClientTest extends TestCase
{
   private String address = "localhost";
   private int port = 6700;
   private Socket socket = null;
   private ObjectOutputStream oos;
   private ObjectInputStream objInputStream;

   public void testClientCall() throws Exception
   {
      getSocket();

      oos.writeObject("This is the request from " + Thread.currentThread().getName());
      oos.reset();
      oos.writeObject(Boolean.TRUE);
      oos.flush();
      oos.reset();

      Object obj = objInputStream.readObject();
      objInputStream.readObject();

      assertEquals("This is response.", obj);

   }

   public void testFailure()
   {
      fail("This is a check to make sure failures reported.");
   }

   private void getSocket() throws IOException
   {
      if(socket == null)
      {
         try
         {
            socket = new Socket(address, port);
            BufferedOutputStream out = new BufferedOutputStream(socket.getOutputStream());
            BufferedInputStream in = new BufferedInputStream(socket.getInputStream());

            oos = new ObjectOutputStream(out);
            objInputStream = new ObjectInputStream(in);
         }
         catch(IOException e)
         {
            e.printStackTrace();
         }
      }
      else
      {
         oos.reset();
         oos.writeByte(1);
         oos.flush();
         oos.reset();
         objInputStream.readByte();
      }
   }
}