/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.jrunit.sample.basic;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;

import junit.framework.TestCase;

/**
 * This is the server for the client/server tests.  It is a very basic implementation
 * for the accepting requests from the client socket and sending a simple string response.
 *
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class SimpleServerTest extends TestCase //ServerTestCase
{

   private int serverBindPort = 6700;
   private int backlog = 200;
   private ServerSocket serverSocket;
   private InetAddress bindAddress;
   private String serverBindAddress = "localhost";
   private int timeout = 5000; // 5 seconds.

   private boolean continueToRun = true; // flag to keep the server listening

   private int requestCounter = 0;

   protected void setUp() throws Exception
   {
      System.out.println("SimpleServerTest::setUp() called.");
      bindAddress = InetAddress.getByName(serverBindAddress);
      serverSocket = new ServerSocket(serverBindPort, backlog, bindAddress);

      // this was done inline since TestCase already has a void parameter run() method
      // so could not create a run() method for the Runnable implementation.
      new Thread()
      {
         public void run()
         {
            try
            {
               startServer();
            }
            catch(Exception e)
            {
               e.printStackTrace();
            }
         }
      }.start();
   }

   public void testRequestCount()
   {
      while(continueToRun)
      {
         try
         {
            Thread.currentThread().sleep(3000);
         }
         catch(InterruptedException e)
         {
            e.printStackTrace();
         }
         System.out.println("Requests taken: " + requestCounter);
      }

      assertEquals(30, requestCounter);
   }

   protected void tearDown() throws Exception
   {
      continueToRun = false;

      System.out.println("Tearing down.  Processed " + requestCounter + " requests");

      if(serverSocket != null && !serverSocket.isClosed())
      {
         serverSocket.close();
         serverSocket = null;
      }
   }

   private void startServer()
   {
      while(continueToRun)
      {
         try
         {
            Socket socket = serverSocket.accept();
            socket.setSoTimeout(timeout);

            BufferedOutputStream bos = new BufferedOutputStream(socket.getOutputStream());
            BufferedInputStream bis = new BufferedInputStream(socket.getInputStream());

            ObjectOutputStream oos = new ObjectOutputStream(bos);
            oos.flush();
            ObjectInputStream objInputStream = new ObjectInputStream(bis);

            processRequest(objInputStream, oos);

            while(continueToRun)
            {
               acknowledge(objInputStream, oos);
               processRequest(objInputStream, oos);
            }
         }
         catch(IOException e)
         {
            System.out.println("Done processing on client socket.");
         }
         catch(ClassNotFoundException e)
         {
         }

      }
   }


   private void processRequest(ObjectInputStream objInputStream, ObjectOutputStream oos)
         throws IOException, ClassNotFoundException
   {
      Object obj = objInputStream.readObject();
      objInputStream.readObject();

      System.out.println("Object read: " + obj);

      Object resp = null;
      try
      {
         boolean interrupted = Thread.interrupted();
         resp = "This is response.";
      }
      catch(Exception ex)
      {
         resp = ex;
      }

      Thread.interrupted(); // clear interrupted state so we don't fail on socket writes

      oos.writeObject(resp);
      oos.reset();
      oos.writeObject(Boolean.TRUE);
      oos.flush();
      oos.reset();

      ++requestCounter;
   }

   private void acknowledge(ObjectInputStream objInputStream, ObjectOutputStream oos) throws IOException
   {
      byte ACK = objInputStream.readByte();
      oos.writeByte(ACK);
      oos.flush();
      oos.reset();
   }


}